'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mongooseMob = exports.getModel = exports.getConnection = exports.Schema = exports.closeAllConnections = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _co = require('co');

var _co2 = _interopRequireDefault(_co);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _asyncExitHook = require('async-exit-hook');

var _asyncExitHook2 = _interopRequireDefault(_asyncExitHook);

var _mongoose = require('mongoose');

var _mongoose2 = _interopRequireDefault(_mongoose);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_mongoose2.default.Promise = global.Promise;

var log = (0, _debug2.default)('mongoose-mob');

var connections = {};

var connect = function connect(uri, opts) {
  log('creating mongoose connection to: ' + uri);
  var connection = _mongoose2.default.createConnection(uri, opts);

  connection.on('connected', function () {
    return log('Mongoose connection open to ' + uri);
  });

  // When the connection is disconnected
  connection.on('disconnected', function () {
    return log('Mongoose ' + uri + ' connection disconnected');
  });

  connection.on('error', function () {
    return log('Mongoose ' + uri + ' connection errored');
  });

  connections[uri] = connection;

  return connection;
};

var connectionOpened = function connectionOpened(connection) {
  return new Promise(function (resolve, reject) {
    try {
      connection.on('connected', function () {
        return resolve();
      });
    } catch (err) {
      reject(err);
    }
  });
};

var closeConnection = function closeConnection(connection, uri) {
  return new Promise(function (resolve, reject) {
    try {
      connection.close(function () {
        log(uri + ' connection disconnected');
        _lodash2.default.unset(connections, uri);
        resolve();
      });
    } catch (err) {
      reject(err);
    }
  });
};

var closeAllConnections = exports.closeAllConnections = function closeAllConnections(callback) {
  log('closing all connections');
  (0, _co2.default)(regeneratorRuntime.mark(function _callee() {
    var uri, connection;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.t0 = regeneratorRuntime.keys(connections);

          case 1:
            if ((_context.t1 = _context.t0()).done) {
              _context.next = 11;
              break;
            }

            uri = _context.t1.value;
            connection = connections[uri];

            if (connection._hasOpened) {
              _context.next = 7;
              break;
            }

            _context.next = 7;
            return connectionOpened(connection);

          case 7:
            _context.next = 9;
            return closeConnection(connection, uri);

          case 9:
            _context.next = 1;
            break;

          case 11:
            callback();

          case 12:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
};

(0, _asyncExitHook2.default)(function (callback) {
  return closeAllConnections(callback);
});

var Schema = exports.Schema = _mongoose2.default.Schema;

var getConnection = exports.getConnection = function getConnection(uri, opts) {
  if (_lodash2.default.isObject(connections[uri])) {
    return connections[uri];
  }

  return connect(uri, opts);
};

var getModel = exports.getModel = function getModel(connection, name, schema) {
  var modelNames = connection.modelNames();
  if (_lodash2.default.includes(modelNames, name)) {
    return connection.model(name);
  } else {
    return connection.model(name, schema);
  }
};

var mongooseMob = exports.mongooseMob = {
  Schema: Schema,
  closeAllConnections: closeAllConnections,
  getConnection: getConnection,
  getModel: getModel
};

exports.default = mongooseMob;